@@ -438,12 +438,6 @@ populate_bar_for_list_view(BjbMainToolbar *self)
 static void
 disconnect_note_handlers (BjbMainToolbarPrivate *priv)
 {
-  if (priv->accel)
-  {
-    gtk_window_remove_accel_group (priv->window, priv->accel);
-    g_clear_pointer (&priv->accel, g_object_unref);
-  }
-
   if (priv->note_renamed != 0)
   {
     g_signal_handler_disconnect (priv->note, priv->note_renamed);
@@ -502,11 +496,11 @@ on_note_color_changed (BijiNoteObj *note, GtkColorButton *button)
 static void
 on_note_content_changed (BjbMainToolbar *self)
 {
-  gchar *str = NULL;
+  const gchar *str = NULL;
   gboolean sensitive = TRUE;
 
   if (self->priv->note)
-    str = biji_note_get_raw_text (self->priv->note);
+    str = biji_note_obj_get_raw_text (self->priv->note);
 
   if (!str || g_strcmp0 (str, "") == 0 || g_strcmp0 (str, "\n") == 0)
     sensitive = FALSE;
@@ -589,7 +583,7 @@ bjb_note_menu_new (BjbMainToolbar *self)
                             G_CALLBACK (webkit_web_view_undo), editor);
   gtk_widget_add_accelerator (item, "activate", priv->accel, GDK_KEY_u,
                              GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
-  gtk_widget_show (item);
+
 
   item = gtk_menu_item_new_with_label (_("Redo"));
   gtk_menu_shell_append (GTK_MENU_SHELL (result), item);
@@ -597,55 +591,60 @@ bjb_note_menu_new (BjbMainToolbar *self)
                             G_CALLBACK (webkit_web_view_redo), editor);
   gtk_widget_add_accelerator (item, "activate", priv->accel, GDK_KEY_r,
                              GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
-  gtk_widget_show (item);
 
   item = gtk_separator_menu_item_new ();
   gtk_menu_shell_append (GTK_MENU_SHELL (result), item);
-  gtk_widget_show (item);
 
-  /* Bullets, ordered list, separator */
-  /* Bullets : unordered list format */
-  item = gtk_menu_item_new_with_label (_("Bullets"));
-  gtk_menu_shell_append (GTK_MENU_SHELL (result), item);
-  g_signal_connect_swapped (item, "activate",
-                            G_CALLBACK (bjb_toggle_bullets), editor);
-  gtk_widget_show (item);
+  if (biji_note_obj_can_format (priv->note))
+  {
 
-  /* Ordered list as 1.mouse 2.cats 3.dogs */
-  item = gtk_menu_item_new_with_label (_("Numbered List"));
-  gtk_menu_shell_append (GTK_MENU_SHELL (result), item);
-  g_signal_connect_swapped (item, "activate",
-                            G_CALLBACK (bjb_toggle_list), editor);
-  gtk_widget_show(item);
+    /* Bullets, ordered list, separator */
+    /* Bullets : unordered list format  */
+    item = gtk_menu_item_new_with_label (_("Bullets"));
+    gtk_menu_shell_append (GTK_MENU_SHELL (result), item);
+    g_signal_connect_swapped (item, "activate",
+                              G_CALLBACK (bjb_toggle_bullets), editor);
 
-  item = gtk_separator_menu_item_new ();
-  gtk_menu_shell_append (GTK_MENU_SHELL (result), item);
-  gtk_widget_show (item);
+
+    /* Ordered list as 1.mouse 2.cats 3.dogs */
+    item = gtk_menu_item_new_with_label (_("Numbered List"));
+    gtk_menu_shell_append (GTK_MENU_SHELL (result), item);
+    g_signal_connect_swapped (item, "activate",
+                              G_CALLBACK (bjb_toggle_list), editor);
+
+
+    item = gtk_separator_menu_item_new ();
+    gtk_menu_shell_append (GTK_MENU_SHELL (result), item);
+
+  }
 
   /* Rename, view tags, separtor */
   item = gtk_menu_item_new_with_label(_("Rename"));
   gtk_menu_shell_append(GTK_MENU_SHELL(result),item);
   g_signal_connect(item,"activate",
                    G_CALLBACK(action_rename_note_callback),self);
-  gtk_widget_show(item);
 
-  item = gtk_menu_item_new_with_label(_("Collections"));
-  gtk_menu_shell_append(GTK_MENU_SHELL(result),item);
-  g_signal_connect(item,"activate",
-                   G_CALLBACK(action_view_tags_callback),self);
-  gtk_widget_show(item);
+
+  if (biji_item_is_collectable (BIJI_ITEM (priv->note)))
+  {
+    item = gtk_menu_item_new_with_label(_("Collections"));
+    gtk_menu_shell_append(GTK_MENU_SHELL(result),item);
+    g_signal_connect(item,"activate",
+                     G_CALLBACK(action_view_tags_callback),self);
+
+  }
 
   item = gtk_separator_menu_item_new ();
   gtk_menu_shell_append (GTK_MENU_SHELL (result), item);
-  gtk_widget_show (item);
+
 
   /* Delete Note */
   item = gtk_menu_item_new_with_label(_("Delete this Note"));
   gtk_menu_shell_append(GTK_MENU_SHELL(result),item);
   g_signal_connect(item,"activate",
                    G_CALLBACK(delete_item_callback),self);
-  gtk_widget_show(item);
 
+  gtk_widget_show_all (result);
   return result;
 }
 
@@ -666,8 +665,6 @@ populate_bar_for_note_view (BjbMainToolbar *self)
     return;
 
   settings = bjb_app_get_settings (g_application_get_default());
-  priv->accel = gtk_accel_group_new ();
-  gtk_window_add_accel_group (priv->window, priv->accel);
 
   /* Go to main view basically means closing note */
   grid = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
@@ -676,11 +673,10 @@ populate_bar_for_note_view (BjbMainToolbar *self)
 
   notes_label = gtk_label_new (_("Notes"));
   gtk_box_pack_start (GTK_BOX (grid), notes_label, TRUE, TRUE, TRUE);
-
-  button = gd_main_toolbar_add_button (bar, NULL, NULL, TRUE);
-
+  button = gtk_button_new ();
   gtk_container_add (GTK_CONTAINER (button), grid);
-
+  gd_main_toolbar_add_widget (bar, button, TRUE);
+ 
   g_signal_connect_swapped (button, "clicked",
                             G_CALLBACK (just_switch_to_main_view), self);
   gtk_widget_add_accelerator (button, "activate", self->priv->accel,
@@ -694,29 +690,32 @@ populate_bar_for_note_view (BjbMainToolbar *self)
 
 
   /* Note Color */
-  if (!biji_note_obj_get_rgba (priv->note, &color))
+  if (biji_item_has_color (BIJI_ITEM (priv->note)))
   {
-    gchar *default_color;
+    if (!biji_note_obj_get_rgba (priv->note, &color))
+    {
+      gchar *default_color;
 
-    g_object_get (G_OBJECT(settings),"color", &default_color, NULL);
-    gdk_rgba_parse (&color, default_color);
-    g_free (default_color);
+      g_object_get (G_OBJECT(settings),"color", &default_color, NULL);
+      gdk_rgba_parse (&color, default_color);
+      g_free (default_color);
+    }
+
+    button = bjb_color_button_new ();
+    gtk_widget_set_tooltip_text (button, _("Note color"));
+    gtk_color_chooser_set_rgba (GTK_COLOR_CHOOSER (button), &color);
+
+    gd_main_toolbar_add_widget (bar, button, FALSE);
+    gtk_widget_set_size_request (gtk_bin_get_child (GTK_BIN (button)),
+                                 COLOR_SIZE, COLOR_SIZE);
+    gtk_widget_show (button);
+
+    g_signal_connect (button, "color-set",
+                      G_CALLBACK (on_color_button_clicked), self);
+    priv->note_color_changed = g_signal_connect (priv->note, "color-changed",
+                               G_CALLBACK (on_note_color_changed), button);
   }
 
-  button = bjb_color_button_new ();
-  gtk_widget_set_tooltip_text (button, _("Note color"));
-  gtk_color_chooser_set_rgba (GTK_COLOR_CHOOSER (button), &color);
-
-  gd_main_toolbar_add_widget (bar, button, FALSE);
-  gtk_widget_set_size_request (gtk_bin_get_child (GTK_BIN (button)),
-                               COLOR_SIZE, COLOR_SIZE);
-  gtk_widget_show (button);
-
-  g_signal_connect (button, "color-set",
-                    G_CALLBACK (on_color_button_clicked), self);
-  priv->note_color_changed = g_signal_connect (priv->note, "color-changed",
-                             G_CALLBACK (on_note_color_changed), button);
-
   /* Sharing */
   priv->share = gd_main_toolbar_add_button (bar, "send-to-symbolic",
                                            NULL, FALSE);
@@ -859,6 +858,8 @@ bjb_main_toolbar_constructed (GObject *obj)
 {
   BjbMainToolbar *self = BJB_MAIN_TOOLBAR (obj);
 
+  self->priv->accel = gtk_accel_group_new ();
+  gtk_window_add_accel_group (self->priv->window, self->priv->accel);
   g_signal_connect_swapped (self->priv->window, "view-changed",
                             G_CALLBACK (populate_main_toolbar), self);
 
@@ -900,6 +901,7 @@ bjb_main_toolbar_finalize (GObject *object)
     priv->search_handler = 0;
   }
 
+  gtk_window_remove_accel_group (priv->window, priv->accel);
   disconnect_note_handlers (priv);
 
   /* chain up */
