@@ -19,28 +19,37 @@
 #include <uuid/uuid.h>
 
 #include "libbiji.h"
+#include "biji-local-note.h" // FIXME !!!! biji_provider_note_new ()
 #include "biji-collection.h"
 
+#include "provider/biji-local-provider.h"
+#include "provider/biji-own-cloud-provider.h"
+
 
 struct _BijiNoteBookPrivate
 {
   /* Notes & Collections */
   GHashTable *items;
+  GHashTable *providers;
 
   /* Signals */
   gulong note_renamed ;
 
   GFile *location;
-  GCancellable *load_cancellable;
+  TrackerSparqlConnection *connection;
+  GdkRGBA color;
 };
 
+
 /* Properties */
 enum {
   PROP_0,
   PROP_LOCATION,
+  PROP_COLOR,
   BIJI_BOOK_PROPERTIES
 };
 
+
 /* Signals */
 enum {
   BOOK_AMENDED,
@@ -50,38 +59,66 @@ enum {
 static guint biji_book_signals[BIJI_BOOK_SIGNALS] = { 0 };
 static GParamSpec *properties[BIJI_BOOK_PROPERTIES] = { NULL, };
 
+
+
 #define BIJI_NOTE_BOOK_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), BIJI_TYPE_NOTE_BOOK, BijiNoteBookPrivate))
 
 G_DEFINE_TYPE (BijiNoteBook, biji_note_book, G_TYPE_OBJECT);
 
+
+
+
+
+
 static void
 biji_note_book_init (BijiNoteBook *self)
 {
-  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, BIJI_TYPE_NOTE_BOOK,
-                                            BijiNoteBookPrivate);
+  BijiNoteBookPrivate *priv;
+  GError *error;
+
+
+  priv = self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, BIJI_TYPE_NOTE_BOOK,
+                                                   BijiNoteBookPrivate);
 
   /* Item path is key for table */
-  self->priv->items = g_hash_table_new_full (g_str_hash,
-                                             g_str_equal,
-                                             NULL,
-                                             g_object_unref);
+  priv->items = g_hash_table_new_full (g_str_hash,
+                                       g_str_equal,
+                                       NULL,
+                                       g_object_unref);
+
+  error = NULL;
+  priv->connection = tracker_sparql_connection_get (NULL, &error);
+
+  if (error)
+  {
+    g_warning ("Tracker db connection failed : %s", error->message);
+    g_error_free (error);
+  }
 }
 
+
+TrackerSparqlConnection *
+biji_note_book_get_tracker_connection (BijiNoteBook *book)
+{
+  return book->priv->connection;
+}
+
+
 static void
 biji_note_book_finalize (GObject *object)
 {
   BijiNoteBook *book = BIJI_NOTE_BOOK (object) ;
 
-  if (book->priv->load_cancellable)
-    g_cancellable_cancel (book->priv->load_cancellable);
 
-  g_clear_object (&book->priv->load_cancellable);
   g_clear_object (&book->priv->location);
   g_hash_table_destroy (book->priv->items);
 
   G_OBJECT_CLASS (biji_note_book_parent_class)->finalize (object);
 }
 
+
+
+
 static void
 biji_note_book_set_property (GObject      *object,
                              guint         property_id,
@@ -89,13 +126,23 @@ biji_note_book_set_property (GObject      *object,
                              GParamSpec   *pspec)
 {
   BijiNoteBook *self = BIJI_NOTE_BOOK (object);
-
+  GdkRGBA *color;
 
   switch (property_id)
     {
     case PROP_LOCATION:
       self->priv->location = g_value_dup_object (value);
       break;
+
+    case PROP_COLOR:
+      color = g_value_get_pointer (value);
+      self->priv->color.red = color->red;
+      self->priv->color.blue = color->blue;
+      self->priv->color.green = color->green;
+      self->priv->color.alpha = color->alpha;
+      g_warning ("book color is not done :%s", gdk_rgba_to_string (&self->priv->color));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -170,15 +217,20 @@ biji_note_book_get_unique_title (BijiNoteBook *book, const gchar *title)
   return new_title;
 }
 
-gboolean
+
+void
 biji_note_book_notify_changed (BijiNoteBook            *book,
                                BijiNoteBookChangeFlag   flag,
                                BijiItem                *item)
 {
-  g_signal_emit (G_OBJECT (book), biji_book_signals[BOOK_AMENDED], 0, flag, item);
-  return FALSE;
+  g_signal_emit (book,
+                 biji_book_signals[BOOK_AMENDED],
+                 0,
+                 flag,
+                 item);
 }
 
+
 /* TODO : use the same for note, put everything there
  * rather calling a func */
 static void
@@ -187,6 +239,7 @@ on_item_deleted_cb (BijiItem *item, BijiNoteBook *book)
   biji_note_book_remove_item (book, item);
 }
 
+
 void
 book_on_note_changed_cb (BijiNoteObj *note, BijiNoteBook *book)
 {
@@ -200,188 +253,113 @@ book_on_item_icon_changed_cb (BijiNoteObj *note, BijiNoteBook *book)
 }
 
 static void
-_biji_note_book_add_one_note (BijiNoteBook *book, BijiNoteObj *note)
+_biji_note_book_add_one_item (BijiNoteBook *book, BijiItem *item)
 {
-  g_return_if_fail (BIJI_IS_NOTE_OBJ (note));
+  g_return_if_fail (BIJI_IS_ITEM (item));
 
-  _biji_note_obj_set_book (note, (gpointer) book);
 
   /* Add it to the list */
   g_hash_table_insert (book->priv->items,
-                       (gpointer) biji_item_get_uuid (BIJI_ITEM (note)),
-                       note);
+                       (gpointer) biji_item_get_uuid (item), item);
 
   /* Notify */
-  g_signal_connect (note, "changed", G_CALLBACK (book_on_note_changed_cb), book);
-  g_signal_connect (note, "renamed", G_CALLBACK (book_on_note_changed_cb), book);
-  g_signal_connect (note, "color-changed", G_CALLBACK (book_on_item_icon_changed_cb), book);
-}
-
-#define ATTRIBUTES_FOR_NOTEBOOK "standard::content-type,standard::name"
-
-static void
-load_location_error (GFile *location,
-                     GError *error)
-{
-  gchar *path = g_file_get_path (location);
-  g_printerr ("Unable to load location %s: %s", path, error->message);
-
-  g_free (path);
-  g_error_free (error);
-}
-
-static void
-release_enum_cb (GObject *source, GAsyncResult *res, gpointer user_data)
-{
-  g_file_enumerator_close_finish (G_FILE_ENUMERATOR (source),
-                                  res,
-                                  NULL);
-  g_object_unref (source);
-}
-
-static void
-create_collection_if_needed (gpointer key,
-                             gpointer value,
-                             gpointer user_data)
-{
-  BijiNoteBook *book = BIJI_NOTE_BOOK (user_data);
-  BijiTrackerInfoSet *set = value;
-  BijiCollection *collection;
-
-  collection = g_hash_table_lookup (book->priv->items, key);
-
-  if (!collection)
+  if (BIJI_IS_NOTE_OBJ (item))
   {
-    collection = biji_collection_new (G_OBJECT (book), key, set->title, set->mtime);
-
-    g_hash_table_insert (book->priv->items,
-                         g_strdup (key),
-                         collection);
-
-    g_signal_connect (collection, "deleted",
-                      G_CALLBACK (on_item_deleted_cb), book);
-    g_signal_connect (collection , "icon-changed",
-                      G_CALLBACK (book_on_item_icon_changed_cb), book);
+    g_signal_connect (item, "changed", G_CALLBACK (book_on_note_changed_cb), book);
+    g_signal_connect (item, "renamed", G_CALLBACK (book_on_note_changed_cb), book);
+    g_signal_connect (item, "color-changed", G_CALLBACK (book_on_item_icon_changed_cb), book);
   }
 
-  /* InfoSet are freed per g_hash_table_destroy thanks to below caller */
+  else if (BIJI_IS_COLLECTION (item))
+  {
+    g_signal_connect (item, "deleted", G_CALLBACK (on_item_deleted_cb), book);
+    g_signal_connect (item , "icon-changed", G_CALLBACK (book_on_item_icon_changed_cb), book);
+  }
 }
 
-static void
-load_book_finish (GHashTable *collections,
-                  gpointer user_data)
-{
-  BijiNoteBook *self = BIJI_NOTE_BOOK (user_data);
-
-  g_hash_table_foreach (collections, create_collection_if_needed, user_data);
-  g_hash_table_destroy (collections);
-
-  biji_note_book_notify_changed (self, BIJI_BOOK_MASS_CHANGE, NULL);
-}
 
 static void
-enumerate_next_files_ready_cb (GObject *source,
-                               GAsyncResult *res,
-                               gpointer user_data)
+on_provider_loaded_cb (BijiProvider *provider,
+                       GList *items,
+                       BijiNoteBook *book)
 {
-  GFileEnumerator *enumerator = G_FILE_ENUMERATOR (source);
-  BijiNoteBook *self;
-  GList *files, *l;
-  GError *error = NULL;
-  gchar *base_path;
+  BijiItem *item = NULL;
+  BijiNoteBookChangeFlag flag = BIJI_BOOK_CHANGE_FLAG;
+  GList *l;
+  gint i = 0;
 
-  files = g_file_enumerator_next_files_finish (enumerator, res, &error);
-  g_file_enumerator_close_async (enumerator, G_PRIORITY_DEFAULT, NULL,
-                                 release_enum_cb, NULL);
 
-  if (error != NULL)
+  for (l=items; l!=NULL; l=l->next)
+  {
+    if (BIJI_IS_ITEM (l->data))
     {
-      load_location_error (g_file_enumerator_get_container (enumerator), error);
-      return;
+      _biji_note_book_add_one_item (book, l->data);
+      i++;
     }
+  }
 
-  self = user_data;
-  base_path = g_file_get_path (self->priv->location);
+  if (i==1)
+    flag = BIJI_BOOK_ITEM_ADDED;
 
-  // now load the notes
-  for (l = files; l != NULL; l = l->next)
-    {
-      GFileInfo *info;
-      const gchar *name;
-      gchar *path;
-      BijiNoteObj *note;
+  else if (i>1)
+    flag = BIJI_BOOK_MASS_CHANGE;
 
-      info = l->data;
-      name = g_file_info_get_name (info);
 
-      if (!g_str_has_suffix (name, ".note"))
-        continue;
-
-      path = g_build_filename (base_path, name, NULL);
-      note = biji_note_get_new_from_file (path);
-
-      _biji_note_book_add_one_note (self, note);
-
-      g_free (path);
-    }
-
-  g_free (base_path);
-  g_list_free_full (files, g_object_unref);
-
-  /* Now we have all notes,
-   * load the collections and we're good to notify loading done */
-  biji_get_all_collections_async (self, load_book_finish, self);
+  if (flag > BIJI_BOOK_CHANGE_FLAG)
+    biji_note_book_notify_changed (book, flag, item);
 }
 
+
+/* 
+ * It should be the right place
+ * to stock somehow providers list
+ * in order to handle properly book__note_new ()
+ * 
+ */
 static void
-enumerate_children_ready_cb (GObject *source,
-                             GAsyncResult *res,
-                             gpointer user_data)
+_add_provider (BijiNoteBook *self,
+               BijiProvider *provider)
 {
-  GFile *location = G_FILE (source);
-  GFileEnumerator *enumerator;
-  GError *error = NULL;
-  BijiNoteBook *self;
+  g_return_if_fail (BIJI_IS_PROVIDER (provider));
 
-  enumerator = g_file_enumerate_children_finish (location,
-                                                 res, &error);
-
-  if (error != NULL)
-    {
-      load_location_error (location, error);
-      return;
-    }
-
-  self = user_data;
-
-  // enumerate all files
-  g_file_enumerator_next_files_async (enumerator, G_MAXINT,
-                                      G_PRIORITY_DEFAULT,
-                                      self->priv->load_cancellable,
-                                      enumerate_next_files_ready_cb,
-                                      self);
+  g_signal_connect (provider, "loaded", 
+                    G_CALLBACK (on_provider_loaded_cb), self);
 }
 
-static void
-note_book_load_from_location (BijiNoteBook *self)
+
+void
+biji_note_book_add_goa_object (BijiNoteBook *self,
+                               GoaObject *object)
 {
-  self->priv->load_cancellable = g_cancellable_new ();
-  g_file_enumerate_children_async (self->priv->location,
-                                   ATTRIBUTES_FOR_NOTEBOOK, 0,
-                                   G_PRIORITY_DEFAULT,
-                                   self->priv->load_cancellable,
-                                   enumerate_children_ready_cb,
-                                   self);
+  BijiProvider *provider;
+  GoaAccount *account;
+  const gchar *type;
+
+  provider = NULL;
+  account =  goa_object_get_account (object);
+
+  if (GOA_IS_ACCOUNT (account))
+  {
+    type = goa_account_get_provider_type (account);
+
+    if (g_strcmp0 (type, "owncloud") ==0)
+      provider = biji_own_cloud_provider_new (self, object);
+  }
+
+  _add_provider (self, provider);
 }
 
+
 static void
 biji_note_book_constructed (GObject *object)
 {
-  BijiNoteBook *self = BIJI_NOTE_BOOK (object);
+  BijiNoteBook *self;
+  BijiProvider *provider;
   gchar *filename;
   GFile *cache;
 
   G_OBJECT_CLASS (biji_note_book_parent_class)->constructed (object);
+  self = BIJI_NOTE_BOOK (object);
 
   /* Ensure cache directory for icons */
   filename = g_build_filename (g_get_user_cache_dir (),
@@ -392,9 +370,11 @@ biji_note_book_constructed (GObject *object)
   g_file_make_directory (cache, NULL, NULL);
   g_object_unref (cache);
 
-  note_book_load_from_location (self);
+  provider = biji_local_provider_new (self, self->priv->location);
+  _add_provider (self, provider);
 }
 
+
 static void
 biji_note_book_class_init (BijiNoteBookClass *klass)
 {
@@ -412,16 +392,37 @@ biji_note_book_class_init (BijiNoteBookClass *klass)
                   G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_POINTER);
 
   properties[PROP_LOCATION] =
-    g_param_spec_object("location",
-                        "The book location",
-                        "The location where the notes are loaded and saved",
-                        G_TYPE_FILE,
-                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
+    g_param_spec_object ("location",
+                         "The book location",
+                         "The location where the notes are loaded and saved",
+                         G_TYPE_FILE,
+                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
+
+
+  properties[PROP_COLOR] =
+    g_param_spec_pointer ("color",
+                         "Default color",
+                         "Note book default color for notes",
+                         G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
+
 
   g_object_class_install_properties (object_class, BIJI_BOOK_PROPERTIES, properties);
   g_type_class_add_private (klass, sizeof (BijiNoteBookPrivate));
 }
 
+
+void
+biji_note_book_get_default_color (BijiNoteBook *book, GdkRGBA *color)
+{
+  g_return_if_fail (BIJI_IS_NOTE_BOOK (book));
+
+  color->red = book->priv->color.red;
+  color->blue = book->priv->color.blue;
+  color->green = book->priv->color.green;
+  color->alpha = book->priv->color.alpha;
+}
+
+
 gboolean 
 biji_note_book_remove_item (BijiNoteBook *book, BijiItem *item)
 {
@@ -440,11 +441,12 @@ biji_note_book_remove_item (BijiNoteBook *book, BijiItem *item)
     /* Signal before doing anything here. So the note is still
      * fully available for signal receiver. */
     biji_note_book_notify_changed (book, BIJI_BOOK_ITEM_TRASHED, to_delete);
-
-    /* Ref note first, hash_table won't finalize it & we can delete it*/
-    g_object_ref (to_delete);
-    g_hash_table_remove (book->priv->items, path);
     biji_item_trash (item);
+    g_hash_table_remove (book->priv->items, path);
+    /* Ref note first, hash_table won't finalize it & we can delete it*/
+    //g_object_ref (to_delete);
+    //g_hash_table_remove (book->priv->items, path);
+    //biji_item_trash (item);
 
     retval = TRUE;
   }
@@ -467,7 +469,7 @@ biji_note_book_add_item (BijiNoteBook *book, BijiItem *item, gboolean notify)
     retval = FALSE;
 
   else if (BIJI_IS_NOTE_OBJ (item))
-    _biji_note_book_add_one_note (book, BIJI_NOTE_OBJ (item));
+    _biji_note_book_add_one_item (book, item);
 
   else if (BIJI_IS_COLLECTION (item))
   {
@@ -498,19 +500,27 @@ biji_note_book_get_item_at_path (BijiNoteBook *book, const gchar *path)
 }
 
 BijiNoteBook *
-biji_note_book_new (GFile *location)
+biji_note_book_new (GFile *location, GdkRGBA *color)
 {
   return g_object_new(BIJI_TYPE_NOTE_BOOK,
                       "location", location,
+                      "color", color,
                       NULL);
 }
 
-BijiNoteObj *
-biji_note_get_new_from_file (const gchar* path)
-{
-  BijiNoteObj* ret = biji_note_obj_new_from_path (path);
 
-  /* The deserializer will handle note type */
+BijiNoteObj *
+biji_note_get_new_from_file (BijiNoteBook *book, const gchar* path)
+{
+  BijiInfoSet  set;
+  BijiNoteObj *ret;
+
+  set.url = (gchar*) path;
+  set.mtime = 0;
+  set.title = NULL;
+  set.content = NULL;
+
+  ret = biji_local_note_new_from_info (book, &set);
   biji_lazy_deserialize (ret);
 
   return ret ;
@@ -533,7 +543,11 @@ get_note_skeleton (BijiNoteBook *book)
 {
   BijiNoteObj *ret = NULL;
   gchar * folder, *name, *path;
+  BijiInfoSet set;
 
+  set.title = NULL;
+  set.content = NULL;
+  set.mtime = 0;
   folder = g_file_get_path (book->priv->location);
 
   while (!ret)
@@ -541,9 +555,10 @@ get_note_skeleton (BijiNoteBook *book)
     name = biji_note_book_get_uuid ();
     path = g_build_filename (folder, name, NULL);
     g_free (name);
+    set.url = path;
 
     if (!g_hash_table_lookup (book->priv->items, path))
-      ret = biji_note_obj_new_from_path (path);
+      ret = biji_local_note_new_from_info (book, &set);
 
     g_free (path);
   }
@@ -553,11 +568,6 @@ get_note_skeleton (BijiNoteBook *book)
   return ret;
 }
 
-static char*
-wrap_note_content (char *content)
-{
-  return g_strdup_printf("<html xmlns=\"http://www.w3.org/1999/xhtml\"><body>%s</body></html>", content);
-}
 
 BijiNoteObj *
 biji_note_book_note_new           (BijiNoteBook *book, gchar *str)
@@ -569,11 +579,11 @@ biji_note_book_note_new           (BijiNoteBook *book, gchar *str)
     gchar *unique, *html;
 
     unique = biji_note_book_get_unique_title (book, str);
-    html = wrap_note_content (str);
+    html = html_from_plain_text (str);
 
     biji_note_obj_set_title (ret, unique);
     biji_note_obj_set_raw_text (ret, str);
-    biji_note_obj_set_html_content (ret, html);
+    biji_note_obj_set_html (ret, html);
 
     g_free (unique);
     g_free (html);
